import dayjs from "dayjs";
import editForm from "../form.vue";
import messageTypeIndex from "../messageType/index.vue";
import { handleTree } from "@/utils/tree";
import { message } from "@/utils/message";
import { ElMessageBox } from "element-plus";
import { usePublicHooks } from "@/hooks/hooks";
import { addDialog } from "@/components/ReDialog";
import { addDrawer } from "@/components/ReDrawer";
import type { FormItemProps } from "./types";
import type { PaginationProps } from "@pureadmin/table";
import { getKeyList, deviceDetection, getPackageSize } from "@pureadmin/utils";
import applicationApi from "@/api/messagePlatform/application";

import { type Ref, reactive, ref, onMounted, h, toRaw, watch } from "vue";
import { useRouter } from "vue-router";

export function useApplication() {
  const form = reactive({
    applicationName: "",
    systemName: "",
    status: "",
    pageNum: 1,
    pageSize: 10
  });
  const curRow = ref();
  const formRef = ref();
  const dataList = ref([]);
  const isShow = ref(false);
  const loading = ref(true);
  const switchLoadMap = ref({});
  const { switchStyle } = usePublicHooks();
  const pagination = reactive<PaginationProps>({
    total: 0,
    pageSize: 10,
    currentPage: 1,
    background: true,
    pageNum: 0
  });
  const router = useRouter();
  const columns: TableColumnList = [
    {
      label: "应用名称",
      prop: "applicationName"
    },
    {
      label: "appId",
      prop: "appId"
    },
    {
      label: "创建人",
      prop: "createBy"
    },
    {
      label: "更新时间",
      prop: "createTime"
    },
    {
      label: "状态",
      cellRenderer: scope => (
        <el-switch
          size={scope.props.size === "small" ? "small" : "default"}
          loading={switchLoadMap.value[scope.index]?.loading}
          v-model={scope.row.status}
          active-value={true}
          inactive-value={false}
          active-text="已启用"
          inactive-text="已停用"
          inline-prompt
          style={switchStyle.value}
          onClick={() => onChange(scope as any)}
        />
      ),
      minWidth: 90
    },
    {
      label: "操作",
      fixed: "right",
      width: 210,
      slot: "operation"
    }
  ];

  function onChange({ row, index }) {
    ElMessageBox.confirm(
      `确认要<strong>${row.status === 0 ? "停用" : "启用"
      }</strong><strong style='color:var(--el-color-primary)'>${row.applicationName
      }</strong>吗?`,
      "系统提示",
      {
        confirmButtonText: "确定",
        cancelButtonText: "取消",
        type: "warning",
        dangerouslyUseHTMLString: true,
        draggable: true
      }
    )
      .then(async () => {
        await applicationApi.update(row);
        message(
          `已${row.status === 0 ? "停用" : "启用"}${row.applicationName}`,
          {
            type: "success"
          }
        );
        onSearch();
      })
      .catch(() => {
        row.status === 0 ? (row.status = 1) : (row.status = 0);
      });
  }

  function handleSizeChange(val: number) {
    console.log(`${val} items per page`);
  }

  function handleCurrentChange(val: number) {
    console.log(`current page: ${val}`);
  }

  function handleSelectionChange(val) {
    console.log("handleSelectionChange", val);
  }

  const handleUpdate = (row?: FormItemProps) => {
    return applicationApi.update(row);
  };

  const handleAdd = (row?: FormItemProps) => {
    return applicationApi.add(row);
  };

  async function handleDelete(row?: FormItemProps) {
    ElMessageBox.confirm(
      `确认要<strong>删除网关名称为</strong><strong style='color:var(--el-color-primary)'>${row.applicationName
      }</strong>吗?`,
      "系统提示",
      {
        confirmButtonText: "确定",
        cancelButtonText: "取消",
        type: "warning",
        dangerouslyUseHTMLString: true,
        draggable: true
      }
    ).then(async () => {
      await applicationApi.remove(row.appId);
      message(`您删除了动态属性名称为${row.applicationName}的这条数据`, {
        type: "success"
      });
      onSearch();
    })
  }

  async function onSearch() {
    loading.value = true;
    const { data } = await applicationApi.fetch(toRaw(form));
    dataList.value = data.records;
    pagination.total = data.total;
    pagination.pageSize = data.pageSize;
    pagination.currentPage = data.pageNum;

    setTimeout(() => {
      loading.value = false;
    }, 500);
  }

  const resetForm = formEl => {
    if (!formEl) return;
    formEl.resetFields();
    onSearch();
  };

  function openDrawer(row) {
    addDrawer({
      title: "消息类型管理",
      size: "90%",
      direction: "ltr",
      props: {
        formInline: {
          appId: row?.appId ?? 0
        }
      },
      contentRenderer: () => h(messageTypeIndex)
    })
  }

  function openDialog(title = "新增", row?: FormItemProps) {
    addDrawer({
      title: `${title}应用`,
      props: {
        formInline: {
          id: row?.id ?? 0,
          applicationName: row?.applicationName ?? "",
          applicationDescribe: row?.applicationDescribe ?? "",
          systemName: row?.systemName ?? "",
          appId: row?.appId ?? "",
          status: row?.status ?? ""
        }
      },
      size: "40%",
      direction: "ltr",
      contentRenderer: () => h(editForm, { ref: formRef, formInline: null }),
      beforeSure: (done, { options }) => {
        const FormRef = formRef.value.getRef();
        const curData = options.props.formInline as FormItemProps;
        function chores() {
          message(
            `您${title}了应用名称为${curData.applicationName}的这条数据`,
            {
              type: "success"
            }
          );
          done(); // 关闭弹框
          onSearch(); // 刷新表格数据
        }
        FormRef.validate(async valid => {
          if (valid) {
            console.log("curData", curData);
            // 表单规则校验通过
            if (title === "新增") {
              // 实际开发先调用新增接口，再进行下面操作
              await handleAdd(curData);
              chores();
            } else {
              await handleUpdate(curData);
              chores();
            }
          }
        });
      }
    });
  }

  /** 高亮当前权限选中行 */
  function rowStyle({ row: { id } }) {
    return {
      cursor: "pointer",
      background: id === curRow.value?.id ? "var(--el-fill-color-light)" : ""
    };
  }

  onMounted(async () => {
    onSearch();
  });
  return {
    form,
    isShow,
    curRow,
    loading,
    columns,
    rowStyle,
    dataList,
    pagination,
    // buttonClass,
    onSearch,
    resetForm,
    openDialog,
    handleDelete,
    handleSizeChange,
    handleCurrentChange,
    handleSelectionChange,
    openDrawer
  };
}
