import dayjs from "dayjs";
import editForm from "../form.vue";
import { handleTree } from "@/utils/tree";
import { message } from "@/utils/message";
import { ElMessageBox } from "element-plus";
import { usePublicHooks } from "@/hooks/hooks";
import { addDialog } from "@/components/ReDialog";
import { addDrawer } from "@/components/ReDrawer";
import type { FormItemProps } from "./types";
import type { PaginationProps } from "@pureadmin/table";
import { getKeyList, deviceDetection } from "@pureadmin/utils";
import dynamicParameterApi from "@/api/messagePlatform/dynamicParameter";
import { type Ref, reactive, ref, onMounted, h, toRaw, watch } from "vue";
import { menuGateWayTypeOptions } from "@/utils/enums/gatewayType";

export function useRole(treeRef: Ref) {
  const form = reactive({
    pageSize: 10,
    pageNum: 1
  });
  const curRow = ref();
  const formRef = ref();
  const dataList = ref([]);
  const treeIds = ref([]);
  const treeData = ref([]);
  const isShow = ref(false);
  const loading = ref(true);
  const isLinkage = ref(false);
  const treeSearchValue = ref();
  const switchLoadMap = ref({});
  const isExpandAll = ref(false);
  const isSelectAll = ref(false);
  const deleteDialogVisible = ref(false);
  const currentDeleteItem = ref<FormItemProps>({
    id: 0,
    name: "",
    parameterDefine: "",
    status: 0,
    createTime: "",
    remark: "",
    createBy: "",
    gatewayType: 0
  });
  const { switchStyle } = usePublicHooks();
  const treeProps = {
    value: "id",
    label: "title",
    children: "children"
  };
  const pagination = reactive<PaginationProps>({
    total: 0,
    pageSize: 10,
    currentPage: 1,
    background: true,
    pageNum: 0
  });
  const columns: TableColumnList = [
    {
      label: "动态参数名称",
      prop: "name"
    },
    {
      label: "网关类型",
      cellRenderer: scope => {
        const { gatewayType } = scope.row;
        const res = menuGateWayTypeOptions.find(
          item => item.value == gatewayType
        );
        return res ? res.label : "";
      }
    },
    {
      label: "创建人",
      prop: "createBy"
    },
    {
      label: "状态",
      cellRenderer: scope => (
        <el-switch
          size={scope.props.size === "small" ? "small" : "default"}
          loading={switchLoadMap.value[scope.index]?.loading}
          v-model={scope.row.status}
          active-value={true}
          inactive-value={false}
          active-text="已启用"
          inactive-text="已停用"
          inline-prompt
          style={switchStyle.value}
          onClick={() => onChange(scope as any)}
        />
      ),
      minWidth: 90
    },
    {
      label: "创建时间",
      prop: "createTime",
      minWidth: 160,
      formatter: ({ createTime }) =>
        dayjs(createTime).format("YYYY-MM-DD HH:mm:ss")
    },
    {
      label: "操作",
      fixed: "right",
      width: 210,
      slot: "operation"
    }
  ];
  // const buttonClass = computed(() => {
  //   return [
  //     "!h-[20px]",
  //     "reset-margin",
  //     "!text-gray-500",
  //     "dark:!text-white",
  //     "dark:hover:!text-primary"
  //   ];
  // });

  function onChange({ row, index }) {
    ElMessageBox.confirm(
      `确认要<strong>${row.status === 0 ? "停用" : "启用"
      }</strong><strong style='color:var(--el-color-primary)'>${row.name
      }</strong>吗?`,
      "系统提示",
      {
        confirmButtonText: "确定",
        cancelButtonText: "取消",
        type: "warning",
        dangerouslyUseHTMLString: true,
        draggable: true
      }
    ).then(async () => {
      await dynamicParameterApi.update(row);
      message(`已${row.status === 0 ? "停用" : "启用"}${row.name}`, {
        type: "success"
      });
    }).catch(() => {
      !row.status ? (row.status = true) : (row.status = false);
    })
  }

  function handleSizeChange(val: number) {
    console.log(`${val} items per page`);
  }

  function handleCurrentChange(val: number) {
    console.log(`current page: ${val}`);
  }

  function handleSelectionChange(val) {
    console.log("handleSelectionChange", val);
  }

  async function onSearch() {
    loading.value = true;
    const {
      data: { records, total, pageSize, pageNum }
    } = await dynamicParameterApi.fetch(toRaw(form));
    dataList.value = records;
    pagination.total = total;
    pagination.pageSize = pageSize;
    pagination.currentPage = pageNum;

    setTimeout(() => {
      loading.value = false;
    }, 500);
  }

  const handleUpdate = (row?: FormItemProps) => {
    return dynamicParameterApi.update(row);
  };

  const handleAdd = (row?: FormItemProps) => {
    return dynamicParameterApi.add(row);
  };

  async function handleDelete(row) {
    ElMessageBox.confirm(
      `确认要<strong>删除动态属性名称为</strong><strong style='color:var(--el-color-primary)'>${row.name
      }</strong>吗?`,
      "系统提示",
      {
        confirmButtonText: "确定",
        cancelButtonText: "取消",
        type: "warning",
        dangerouslyUseHTMLString: true,
        draggable: true
      }
    ).then(async () => {
      await dynamicParameterApi.remove(row.id);
      message(`您删除了动态属性名称为${row.name}的这条数据`, {
        type: "success"
      });
      onSearch();
    })
  }

  /** 高亮当前权限选中行 */
  function rowStyle({ row: { id } }) {
    return {
      cursor: "pointer",
      background: id === curRow.value?.id ? "var(--el-fill-color-light)" : ""
    };
  }
  const handleOpenDeleteModal = (row: FormItemProps) => {
    currentDeleteItem.value = row;
    deleteDialogVisible.value = true;
  };

  const resetForm = formEl => {
    if (!formEl) return;
    formEl.resetFields();
    onSearch();
  };

  function openDialog(title = "新增", row?: FormItemProps) {
    addDrawer({
      title: `${title}动态参数`,
      direction: "ltr",
      props: {
        formInline: {
          id: row?.id ?? 0,
          name: row?.name ?? "",
          parameterDefine: row?.parameterDefine ?? "",
          gatewayType: row?.gatewayType ?? "",
          status: row?.status ?? "",
          createTime: row?.createTime ?? "",
          remark: row?.remark ?? "",
          createBy: row?.createBy ?? "",
          attrList: row?.parameterDefine ? JSON.parse(row.parameterDefine) : []
        }
      },
      size: "80%",
      contentRenderer: () => h(editForm, { ref: formRef, formInline: null }),
      beforeSure: (done, { options }) => {
        const FormRef = formRef.value.getRef();
        const curData = options.props.formInline as FormItemProps;
        curData.parameterDefine = JSON.stringify(curData.attrList);
        console.log("最终值", curData);
        function chores() {
          message(`您${title}了动态参数名称为${curData.name}的这条数据`, {
            type: "success"
          });
          done(); // 关闭弹框
          onSearch(); // 刷新表格数据
        }
        FormRef.validate(async valid => {
          if (valid) {
            console.log("curData", curData);
            // 表单规则校验通过
            if (title === "新增") {
              // 实际开发先调用新增接口，再进行下面操作
              await handleAdd(curData);
              chores();
            } else {
              await handleUpdate(curData);
              chores();
            }
          }
        });
      }
    });
  }

  const onQueryChanged = (query: string) => {
    treeRef.value!.filter(query);
  };

  onMounted(async () => {
    onSearch();
  });

  watch(isExpandAll, val => {
    val
      ? treeRef.value.setExpandedKeys(treeIds.value)
      : treeRef.value.setExpandedKeys([]);
  });

  watch(isSelectAll, val => {
    val
      ? treeRef.value.setCheckedKeys(treeIds.value)
      : treeRef.value.setCheckedKeys([]);
  });

  return {
    form,
    isShow,
    curRow,
    loading,
    columns,
    dataList,
    treeData,
    treeProps,
    rowStyle,
    isLinkage,
    pagination,
    isExpandAll,
    isSelectAll,
    treeSearchValue,
    // buttonClass,
    onSearch,
    resetForm,
    openDialog,
    handleDelete,
    onQueryChanged,
    // handleDatabase,
    handleSizeChange,
    handleCurrentChange,
    handleSelectionChange,
    deleteDialogVisible,
    handleOpenDeleteModal,
    currentDeleteItem
  };
}
