import dayjs from "dayjs";
import editForm from "../form.vue";
import { handleTree } from "@/utils/tree";
import { message } from "@/utils/message";
import { ElMessageBox } from "element-plus";
import { usePublicHooks } from "@/hooks/hooks";
import { addDialog } from "@/components/ReDialog";
import { addDrawer } from "@/components/ReDrawer";
import type { FormItemProps } from "./types";
import type { PaginationProps } from "@pureadmin/table";
import { getKeyList, deviceDetection } from "@pureadmin/utils";
import smsTemplateApi from "@/api/messagePlatform/smsTemplate";
import { useMessagePlatform } from "@/store/modules/messagePlatform"
import { templateTypeOptions } from "./enums";
import { type Ref, reactive, ref, onMounted, h, toRaw, watch } from "vue";

export function useSmsTemplate() {
  const form = reactive({
    messageGatewayId: "",
    status: "",
    templateContent: "",
    pageNum: 1,
    pageSize: 10
  });
  const curRow = ref();
  const formRef = ref();
  const dataList = ref([]);
  const isShow = ref(false);
  const loading = ref(true);
  const switchLoadMap = ref({});
  const { switchStyle } = usePublicHooks();
  const pagination = reactive<PaginationProps>({
    total: 0,
    pageSize: 10,
    currentPage: 1,
    background: true,
    pageNum: 0
  });
  const columns: TableColumnList = [
    {
      label: "模板代码",
      prop: "templateCode"
    },
    {
      label: "模板名称",
      prop: "templateName"
    },
    {
      label: "模板内容",
      prop: "templateContent"
    },
    {
      label: "模板类型",
      prop: "templateType",
      cellRenderer: scope => {
        const option = templateTypeOptions.find(
          item => item.value == scope.row.templateType
        );
        return option ? option.label : "";
      }
    },
    {
      label: "运行商",
      prop: "messageGatewayId",
      cellRenderer: scope => {
        const option = useMessagePlatform().messageGatewayList.find(
          item => item.id == scope.row.messageGatewayId
        );
        return option ? option.gatewayName : "";
      }
    },
    {
      label: "状态",
      cellRenderer: scope => (
        <el-switch
          size={scope.props.size === "small" ? "small" : "default"}
          loading={switchLoadMap.value[scope.index]?.loading}
          v-model={scope.row.status}
          active-value={true}
          inactive-value={false}
          active-text="已启用"
          inactive-text="已停用"
          inline-prompt
          style={switchStyle.value}
          onChange={() => onChange(scope as any)}
        />
      ),
      minWidth: 90
    },
    {
      label: "操作",
      fixed: "right",
      width: 210,
      slot: "operation"
    }
  ];
  // const buttonClass = computed(() => {
  //   return [
  //     "!h-[20px]",
  //     "reset-margin",
  //     "!text-gray-500",
  //     "dark:!text-white",
  //     "dark:hover:!text-primary"
  //   ];
  // });

  function onChange({ row, index }) {
    ElMessageBox.confirm(
      `确认要<strong>${row.status === 0 ? "停用" : "启用"
      }</strong><strong style='color:var(--el-color-primary)'>${row.templateName
      }</strong>吗?`,
      "系统提示",
      {
        confirmButtonText: "确定",
        cancelButtonText: "取消",
        type: "warning",
        dangerouslyUseHTMLString: true,
        draggable: true
      }
    )
      .then(async () => {
        await smsTemplateApi.update(row.id)
        message(`已${row.status === 0 ? "停用" : "启用"}${row.templateName}`, {
          type: "success"
        });
      })
      .catch(() => {
        !row.status ? (row.status = true) : (row.status = false);
      });
  }


  function handleSizeChange(val: number) {
    console.log(`${val} items per page`);
  }

  function handleCurrentChange(val: number) {
    console.log(`current page: ${val}`);
  }

  function handleSelectionChange(val) {
    console.log("handleSelectionChange", val);
  }

  async function onSearch() {
    loading.value = true;
    const { data } = await smsTemplateApi.fetch(toRaw(form));
    dataList.value = data.records;
    pagination.total = data.total;
    pagination.pageSize = data.pageSize;
    pagination.currentPage = data.pageNum;
    setTimeout(() => {
      loading.value = false;
    }, 500);
  }



  const resetForm = formEl => {
    if (!formEl) return;
    formEl.resetFields();
    onSearch();
  };
  const handleUpdate = (row?: FormItemProps) => {
    return smsTemplateApi.update(row);
  };

  const handleAdd = (row?: FormItemProps) => {
    return smsTemplateApi.add(row);
  };

  async function handleDelete(row?: FormItemProps) {
    ElMessageBox.confirm(
      `确认要<strong>删除短信模板名称为</strong><strong style='color:var(--el-color-primary)'>${row.templateName}</strong>吗?`,
      "系统提示",
      {
        confirmButtonText: "确定",
        cancelButtonText: "取消",
        type: "warning",
        dangerouslyUseHTMLString: true,
        draggable: true
      }
    ).then(async () => {
      await smsTemplateApi.remove(row.id);
      message(`您删除了短信模板名称为${row.templateName}的这条数据`, {
        type: "success"
      });
      onSearch();
    });
  }

  function openDialog(title = "新增", row?: FormItemProps) {
    addDrawer({
      title: `${title}短信模板`,
      size: "45%",
      direction: "ltr",
      props: {
        formInline: {
          id: row?.id ?? 0,
          /** 模板名称 */
          templateName: row?.templateName ?? "",
          /** 模板编号 */
          templateCode: row?.templateCode ?? "",
          /** 模板内容 */
          templateContent: row?.templateContent ?? "",
          /**  0-通用、1-验证码 */
          templateType: row?.templateType ?? "",
          /** 运营商-对应消息网关id */
          messageGatewayId: row?.messageGatewayId ?? "",
          /** 状态 */
          status: row?.status ?? ""
        }
      },
      closeOnClickModal: false,
      contentRenderer: () => h(editForm, { ref: formRef, formInline: null }),
      beforeSure: (done, { options }) => {
        const FormRef = formRef.value.getRef();
        const curData = options.props.formInline as FormItemProps;
        function chores() {
          message(`您${title}了短信模板名称为${curData.templateName}的这条数据`, {
            type: "success"
          });
          done(); // 关闭弹框
          onSearch(); // 刷新表格数据
        }
        FormRef.validate(async valid => {
          if (valid) {
            console.log("curData", curData);
            // 表单规则校验通过
            if (title === "新增") {
              // 实际开发先调用新增接口，再进行下面操作
              await handleAdd(curData);
              chores();
            } else {
              await handleUpdate(curData);
              chores();
            }
          }
        });
      }
    });
  }

  /** 高亮当前权限选中行 */
  function rowStyle({ row: { id } }) {
    return {
      cursor: "pointer",
      background: id === curRow.value?.id ? "var(--el-fill-color-light)" : ""
    };
  }


  onMounted(async () => {
    onSearch();
    useMessagePlatform().reqMessageGatewayList()
  });
  return {
    form,
    isShow,
    loading,
    columns,
    rowStyle,
    dataList,
    pagination,
    onSearch,
    resetForm,
    openDialog,
    handleDelete,
    // handleDatabase,
    handleSizeChange,
    handleCurrentChange,
    handleSelectionChange
  };
}
